// This is a reduced/simplified version of RobotTracke_v4.0
import java.util.*;
import java.io.*;
import java.sql.Time;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.imageio.*;

public class RobotTrackerMapOnlyVersion extends JFrame {
	private static  boolean 	USE_WEB_CAM = false;		// Set to true if webcam is available (false for home version)
	private static  boolean 	USE_BLUETOOTH = false;	// Set to true if bluetooth is available (false for home version)

	public  static final int 		CAMERA_WIDTH = 640;
	public  static final int 		CAMERA_HEIGHT = 480;

	public  static final float 		VERSION = 4.0f;	// Current Version
	public  static final int 		YEAR = 2011;		// Year of Current Version

    public  static final int 		MIN_WIDTH  = CAMERA_WIDTH + 69;  // Offset the size of the window by the image size
    public  static final int 		MIN_HEIGHT = CAMERA_HEIGHT + 73;  // Offset the size of the window by the image size
    private static final int		FPS_TRAIL = 10;						// number of previous fps values to average from




    // Image icons used in the status bar and menus
	private static final ImageIcon	CAMERA_ICON = new ImageIcon("ButtonCamera.png");

	private static final Color		GREEN = new Color(0,100,0);
	private static final Color		RED = new Color(200,0,0);
	private static final Color		GRAY = new Color(100,100,100);


	// Used on the bottom panel to display information
    private JLabel 				locationFieldX, locationFieldY;
    public  TrackingPanel 		trackingPanel;


    // Dialog boxes used i this application
	private MapDialog			mapDialog;


	// Snapshot and Movie Clip variables
 	private Image 				photo;							// tracker image
	private int 				snapshotNumber = 0;				// the next manual snapshot number ... increases each time
	private javax.swing.Timer 	snapTimer;						// timer fortaking snapshots
	private	long 				prevTime = new Date().getTime();// used to keep track of previous time
	private	int[] 				fpsTimes;						// used to keep track of average frames per second
	private	int 				fpsCount;						// used to keep track of index into fpsTimes circular array
    private int 				snapshotCount = 0;				// number of snapshots taken
    private boolean 			recordSnapshots = false;		// True when recording snapshots (for movies)
    private long 				start;							// start time for taking snapshots

	// Used for tracking
    public  int 			frameRate = 9;  					// will be loaded from config file
 	public  boolean			combineMultipleTrackers = false;	// will be loaded from config file
 	public  int 			stationID = 0;   					// will be loaded from config file
 	public  String[]		stationAddresses;   				// will be loaded from config file
 	public  boolean 		tracking = false;					// True if currently tracking robots
 	public  String			workingDirectory = ".";				// Folder that contains planner, snapshots, traces, etc..
    private	int				bluetoothPort = 1;			// The com port that contains the bluetooth device
	private boolean	 		enableRobotCommunication = true;	// Whether or not the program will communicate with the robot
 	private TreeMap<Integer, String>	robotAddresses;				// The bluetooth addresses of the robots

	// Used for Mapping
	public  boolean			mapping;				// tru iof showing map, false otherwise
	public  int				errorSetting;			// error type (raw, error, angle error, distance error, full Gaussian
	public  int 			resolution;				// sigma error (x1, x2,x3, x5, or x10)
    public  DataSet[]		dataSets;				// sonar, dirrs and side IR
    private MapRenderer 	renderer;				// the map-rendering object


    public RobotTrackerMapOnlyVersion(String frameTitle) {
        super(frameTitle);

		// Keep track of frames per second
		fpsCount = 0;
		fpsTimes = new int[FPS_TRAIL];

		// Map-related settings
		mapping = !USE_WEB_CAM;
		errorSetting = 0;
		resolution = 0;
		dataSets = new DataSet[0];	// No data yet, must be loaded

        // Set-up tracking panel
        trackingPanel = new TrackingPanel(this);
        getContentPane().add(trackingPanel, BorderLayout.CENTER);

        // Load up the configuaration file
        loadConfigFile();

        // Setup the Menubar and Toolbar
        setupToolBar();

 		// Event Handler for Window Closing
        addWindowListener(new WindowAdapter() {
        	public void windowClosing(WindowEvent e) {
        		closeProgram();
    		}
    	});

        setSize(MIN_WIDTH, MIN_HEIGHT);
        setResizable(false);

		mapDialog = new MapDialog(this);
		if (!USE_WEB_CAM) {
			setSize(MIN_WIDTH+TrackingPanel.MAP_BORDER_SIZE*2, MIN_HEIGHT+TrackingPanel.MAP_BORDER_SIZE*2);
			mapDialog.setVisible(true);
		}
		renderer = new MapRenderer(this);
    }

 	// Set up the toolbar to appear on the left side of the window
	private void setupToolBar() {
		JToolBar toolbar;
		JButton  snapshotButton;

		// Create a vertical toolbar
        toolbar = new JToolBar(JToolBar.VERTICAL);
        toolbar.setFloatable(false);

		// Create a panel to hold the toolbar
        JPanel panel = new JPanel();
        panel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER,0,0));
        getContentPane().add(panel, java.awt.BorderLayout.WEST);
        panel.add(toolbar);

        // Create the toolbar with all the necessary buttons
	    toolbar.add(snapshotButton = new JButton(CAMERA_ICON));
        snapshotButton.setBackground(Color.white);
        snapshotButton.setToolTipText("Take a screenshot");
	    snapshotButton.addActionListener(new ActionListener() {
	        public void actionPerformed (ActionEvent e) {
	        	handleSnapshotButtonPress();
	        }
	    });

        toolbar.addSeparator();
        toolbar.add(new JSeparator());
        //toolbar.add(new JLabel(" Mouse"));
	    toolbar.add(locationFieldX = new JLabel("   x: 0"));
	    toolbar.add(locationFieldY = new JLabel("   y: 0"));

    }

    // Update the status bar with the given text
    public void updateStatusBar(String s, Color c) {
    }

	// Event handler for when the snapshot button is pressed ... bring up a dialog box
	private void handleSnapshotButtonPress() {
		if (!mapping && photo == null) {
			updateStatusBar("Error: No image available ... snapshot not saved", Color.red.darker());
			return;
		}
		FileDialog saveDialog = new FileDialog(this, "Save PNG", FileDialog.SAVE);
		saveDialog.setDirectory(workingDirectory);
		saveDialog.setFile("Snapshot" + snapshotNumber + ".png");
		saveDialog.setVisible(true);

     	String directory = saveDialog.getDirectory();
        String filename = saveDialog.getFile();

		if (filename == null)
			updateStatusBar("Error: cannot find working directory \"" + workingDirectory + "\"", Color.red.darker());
        else {
        	BufferedImage bi;
        	if (!mapping)
				bi = new BufferedImage(photo.getWidth(null), photo.getHeight(null), BufferedImage.TYPE_INT_RGB);
			else
				bi = new BufferedImage(trackingPanel.mapImage.getWidth(null), trackingPanel.mapImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
	        Graphics2D g2 = bi.createGraphics();

	        // Decide whether or not to display tags

	        if (mapping) {
	        	// Show a shaded background while creating map
				g2.setColor(Color.LIGHT_GRAY);
				g2.fillRect(0, 0, trackingPanel.mapImage.getWidth(null), trackingPanel.mapImage.getHeight(null));
	        	if (!trackingPanel.hideImage)
	        		g2.drawImage(trackingPanel.currentImage, TrackingPanel.MAP_BORDER_SIZE, TrackingPanel.MAP_BORDER_SIZE, this);
	        	g2.drawImage(trackingPanel.mapImage, 0, 0, this);
	        }
	        else {
	        	if (trackingPanel.showTags)
	        		g2.drawImage((Image)trackingPanel.getModifiedImage(), null, null);
	   			else
	        		g2.drawImage((Image)trackingPanel.getImage(), null, null);
	        }

			// Decide whether or not to display paths
	        if (trackingPanel.showDesiredPath)
				trackingPanel.drawUserDefinedPath(g2);
			if (trackingPanel.showEstimatedPath)
				trackingPanel.drawEstimatePath(g2);
			if(trackingPanel.showActualPath)
				trackingPanel.drawActualPath(g2);

			if (mapping) {
				g2.setColor(Color.BLACK);
        		g2.drawRect(TrackingPanel.MAP_BORDER_SIZE,TrackingPanel.MAP_BORDER_SIZE,640,480);
        		g2.setColor(Color.WHITE);
        		g2.drawRect(TrackingPanel.MAP_BORDER_SIZE-1,TrackingPanel.MAP_BORDER_SIZE-1,642,482);
			}


			// Save it
	        try {
	       		ImageIO.write(bi, "png", new File(directory + filename));
	       		snapshotNumber++;
	        }
	        catch (Exception ex) {
	        	JOptionPane.showMessageDialog(null, "Error saving snapshot", "Error", JOptionPane.ERROR_MESSAGE);
			}
        }
    }

	// Saves the image as a PNG in file
	private void storeSnapshot(Image photo, String fileName) {
		if (photo == null)
			return;
		BufferedImage bi = new BufferedImage(photo.getWidth(null), photo.getHeight(null), BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = bi.createGraphics();
        g2.drawImage(photo, null, null);
        try {
       		ImageIO.write(bi, "png", new File(fileName));
        }
        catch (Exception ex) {
        	System.out.println("Error: Cannot save TrackerImage.png to file");
		}
	}

    // Load configuration file
	private void loadConfigFile() {
		BufferedReader in = null;
		String s = "";
		stationAddresses = new String[3];
		try {
            in = new BufferedReader(new FileReader("RobotTrackerConfig.txt"));
            s = in.readLine();
            	frameRate = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	combineMultipleTrackers = Boolean.parseBoolean(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	stationID = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	stationAddresses[0] = s.substring(s.indexOf('=')+2).trim();
            s = in.readLine();
            	stationAddresses[1] = s.substring(s.indexOf('=')+2).trim();
            s = in.readLine();
            	stationAddresses[2] = s.substring(s.indexOf('=')+2).trim();
            s = in.readLine();
            	trackingPanel.diff = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.sum = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.showActualPath = Boolean.parseBoolean(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.showDesiredPath = Boolean.parseBoolean(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.showEstimatedPath = Boolean.parseBoolean(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            s = s.substring(s.indexOf('=')+2);
				String[] cols = s.split(",");
            	trackingPanel.actualPathColor = new Color(Integer.parseInt(cols[0]), Integer.parseInt(cols[1]), Integer.parseInt(cols[2]));
            s = in.readLine();
            s = s.substring(s.indexOf('=')+2);
				cols = s.split(",");
            	trackingPanel.desiredPathColor = new Color(Integer.parseInt(cols[0]), Integer.parseInt(cols[1]), Integer.parseInt(cols[2]));
            s = in.readLine();
            s = s.substring(s.indexOf('=')+2);
				cols = s.split(",");
            	trackingPanel.estimatePathColor = new Color(Integer.parseInt(cols[0]), Integer.parseInt(cols[1]), Integer.parseInt(cols[2]));
            s = in.readLine();
            s = s.substring(s.indexOf('=')+2);
				cols = s.split(",");
            	trackingPanel.tagColor = new Color(Integer.parseInt(cols[0]), Integer.parseInt(cols[1]), Integer.parseInt(cols[2]));
				trackingPanel.tagColorIsDark = (((trackingPanel.tagColor.getRed() * 299) + (trackingPanel.tagColor.getGreen() * 587) + (trackingPanel.tagColor.getBlue() * 114)) / 1000) < 125;
            s = in.readLine();
            	trackingPanel.showTags = Boolean.parseBoolean(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.trackID = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.xOffset = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	trackingPanel.yOffset = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	bluetoothPort = Integer.parseInt(s.substring(s.indexOf('=')+2));
            s = in.readLine();
            	workingDirectory = s.substring(s.indexOf('=')+2);
            s = in.readLine();
            	enableRobotCommunication = Boolean.parseBoolean(s.substring(s.indexOf('=')+2));
            s = in.readLine();

			// Read in the Robot ID/BluetoothAddress pairs
            String data = s.substring(s.indexOf('=')+2);
			if (data.startsWith("{") && data.substring(data.length()-1).compareTo("}")==0) {
				data = data.substring(1,data.length()-1);
				data = data.replace("[","");
				String[] pairs = data.split("]");
				robotAddresses = new TreeMap<Integer, String>();
				for (int i=0;i<pairs.length;i++){
					String[] pair = pairs[i].split(",");
					Integer key = new Integer(pair[0]);
					String value = new String(pair[1]);
					robotAddresses.put(key, value);
				}
			}
			in.close();
        } catch (Exception e) {
        	e.printStackTrace();
        	JOptionPane.showMessageDialog(null, "Problem reading configuration file RobotTrackerConfig.txt", "Error", JOptionPane.ERROR_MESSAGE);
        } finally {
        	try { in.close(); } catch (Exception e) {}
        }
	}

	// Save configuration file
 	public void saveConfigFile() {
		try {
            PrintWriter out = new PrintWriter(new FileWriter("RobotTrackerConfig.txt"));
            out.println("FrameRate           = " + frameRate);
			out.println("UseMultipleTrackers = " + combineMultipleTrackers);
			out.println("StationID           = " + stationID);
			out.println("StationAddress1     = " + stationAddresses[0]);
			out.println("StationAddress2     = " + stationAddresses[1]);
			out.println("StationAddress3     = " + stationAddresses[2]);
			out.println("TrackDifference     = " + trackingPanel.diff);
            out.println("TrackSum            = " + trackingPanel.sum);
			out.println("ShowActualPath      = " + trackingPanel.showActualPath);
            out.println("ShowDesiredPath     = " + trackingPanel.showDesiredPath);
			out.println("ShowEstimatedPath   = " + trackingPanel.showEstimatedPath);
			out.println("ActualPathColor     = " + trackingPanel.actualPathColor.getRed() + "," + trackingPanel.actualPathColor.getGreen() + "," + trackingPanel.actualPathColor.getBlue());
			out.println("DesiredPathColor    = " + trackingPanel.desiredPathColor.getRed() + "," + trackingPanel.desiredPathColor.getGreen() + "," + trackingPanel.desiredPathColor.getBlue());
			out.println("EstimatePathColor   = " + trackingPanel.estimatePathColor.getRed() + "," + trackingPanel.estimatePathColor.getGreen() + "," + trackingPanel.estimatePathColor.getBlue());
			out.println("TagColor            = " + trackingPanel.tagColor.getRed() + "," + trackingPanel.tagColor.getGreen() + "," + trackingPanel.tagColor.getBlue());
            out.println("ShowTags            = " + trackingPanel.showTags);
			out.println("TrackID             = " + trackingPanel.trackID);
			out.println("XOffset             = " + trackingPanel.xOffset);
			out.println("YOffset             = " + trackingPanel.yOffset);
			out.println("BluetoothPort       = " + bluetoothPort);
			out.println("WorkingDirectory    = " + workingDirectory);
			out.println("RobotCommunication  = " + enableRobotCommunication);
			out.print(  "RobotAddresses      = {");
			Object[] keys = robotAddresses.keySet().toArray();
			for (int i=0;i<robotAddresses.keySet().size();i++) {
				out.print("["+keys[i]+","+robotAddresses.get(keys[i])+"]");
			}
			out.println("}");
			out.println();
            out.close();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Cannot write configuration file RobotTrackerConfig.txt", "Error", JOptionPane.ERROR_MESSAGE);
        }
	}


	// Called by tracking panel to display mouse location
	public void setMouseLoc(Point p) {
		if (combineMultipleTrackers)
			p.translate(0, -trackingPanel.yOffset);
		if (mapping)
			p.translate(-trackingPanel.MAP_BORDER_SIZE,-trackingPanel.MAP_BORDER_SIZE);
		//locationField.setText(String.format("%11s","(" + p.x + "," + (CAMERA_HEIGHT - p.y - 1) + ")"));
		locationFieldX.setText(String.format("x: %d",p.x));
		locationFieldY.setText(String.format("y: %d",(CAMERA_HEIGHT - p.y - 1)));
	}

	// Do what needs to be done whent he program is closed
	private void closeProgram(){
        saveConfigFile();
		System.exit(1);
	}

	// Erase the map data
	public void eraseMap() {
		dataSets = new DataSet[0];
		reDrawMap();
	}

	// Called to re-draw the map when necessary
	public void reDrawMap() {
		Thread t = new Thread(renderer);
		t.setPriority(1);
		t.start();
	}

	// Called after the relative sensor weights have been changed, to redraw the map
	public void reWeightMap() {
		Thread t = new Thread() {
			public void run() {
				trackingPanel.setMaxProgress(2);
				trackingPanel.setProgress(0);
				trackingPanel.setProgressMessage("Fusing sensor data..");
				renderer.fuseSensorData();
				trackingPanel.advance();
				renderer.computeMap();
				trackingPanel.advance();
			}
		};
		t.setPriority(1);
		t.start();
	}

	// Called to redraw  the map after a color change has been made
	public void reColorMap() {
		Thread t = new Thread() {
			public void run() {
				trackingPanel.setMaxProgress(1);
				trackingPanel.setProgress(0);
				trackingPanel.setProgressMessage("Applying color map..");
				renderer.computeMap();
				trackingPanel.advance();
			}
		};
		t.setPriority(1);
		t.start();
	}

	public void update() {
		repaint();
	}


	// This is where it all starts ...
    public static void main(String[] args) {
    	RobotTrackerMapOnlyVersion tracker = null;
		try {
			tracker = new RobotTrackerMapOnlyVersion("Robot Tracker v" + VERSION + " - Mapping Only");
			tracker.setVisible(true);
        }
        catch (Exception ex) {
            JOptionPane.showMessageDialog(null, "Unexpected Error", "Error", JOptionPane.ERROR_MESSAGE);
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }
}